import { Test, TestingModule } from '@nestjs/testing';
import { Job } from 'bull';
import { ReposService } from '../repos/repos.service';
import { PipelineUnits } from './enums/pipeline-units.enum';
import { PipelineTaskConsumer } from './pipeline-task.consumer';
import { PipelineTask } from './pipeline-task.entity';
import { PipelineTasksService } from './pipeline-tasks.service';

describe('PipelineTaskConsumer', () => {
  let consumer: PipelineTaskConsumer;
  let tasksService: PipelineTasksService;
  const getJob = () =>
    ({
      data: {
        pipelineId: 'test',
        units: [PipelineUnits.checkout, PipelineUnits.test],
      },
    } as Job<PipelineTask>);

  beforeEach(async () => {
    const module: TestingModule = await Test.createTestingModule({
      providers: [
        {
          provide: PipelineTasksService,
          useValue: {
            doNextTask: () => undefined,
          },
        },
        {
          provide: ReposService,
          useValue: {
            getWorkspaceRootByTask: () => 'workspace-root',
          },
        },
        PipelineTaskConsumer,
      ],
    }).compile();

    tasksService = module.get(PipelineTasksService);
    consumer = module.get(PipelineTaskConsumer);
  });

  it('should be defined', () => {
    expect(consumer).toBeDefined();
  });

  describe('onCompleted', () => {
    it('should call doNextTask()', () => {
      const job = getJob();
      const doNextTask = jest.spyOn(tasksService, 'doNextTask');
      consumer.onCompleted(job);
      expect(doNextTask).toHaveBeenCalledTimes(1);
    });
  });

  describe('doTask', () => {
    it('should do all task', () => {});
  });
});
